@@ -53,9 +53,9 @@ class EB_MATLAB(EasyBlock):
 
     @staticmethod
     def extra_options():
-        extra_vars = [
-                      ('java_options', ['-Xmx256m', "$_JAVA_OPTIONS value set for install and in module file.", CUSTOM]),
-                     ]
+        extra_vars = {
+            'java_options': ['-Xmx256m', "$_JAVA_OPTIONS value set for install and in module file.", CUSTOM],
+        }
         return EasyBlock.extra_options(extra_vars)
 
     def configure_step(self):
